@@ -24,31 +24,37 @@ def domain(value: str, /, *, rfc_1034: bool = False, rfc_2782: bool = False):
         value:
             Domain string to validate.
         rfc_1034:
-            Allow trailing dot in domain name.
+            Allows optional trailing dot in the domain name.
             Ref: [RFC 1034](https://www.rfc-editor.org/rfc/rfc1034).
         rfc_2782:
             Domain name is of type service record.
+            Allows optional underscores in the domain name.
             Ref: [RFC 2782](https://www.rfc-editor.org/rfc/rfc2782).
 
 
     Returns:
         (Literal[True]): If `value` is a valid domain name.
         (ValidationError): If `value` is an invalid domain name.
+
+    Raises:
+        (UnicodeError): If `value` cannot be encoded into `idna` or decoded into `utf-8`.
     """
     if not value:
         return False
     try:
         return not re.search(r"\s", value) and re.match(
             # First character of the domain
-            rf"^(?:[a-zA-Z0-9{'_'if rfc_2782 else ''}]"
-            # Sub domain + hostname
-            + r"(?:[a-zA-Z0-9-_]{0,61}[A-Za-z0-9])?\.)"
+            rf"^(?:[a-z0-9{r'_?'if rfc_2782 else ''}]"
+            # Sub-domain
+            + rf"(?:[a-z0-9-{r'_?'if rfc_2782 else ''}]{{0,61}}"
+            # Hostname
+            + rf"[a-z0-9{r'_?'if rfc_2782 else ''}])?\.)"
             # First 61 characters of the gTLD
-            + r"+[A-Za-z0-9][A-Za-z0-9-_]{0,61}"
+            + r"+[a-z0-9][a-z0-9-_]{0,61}"
             # Last character of the gTLD
-            + rf"[A-Za-z]{r'.$' if rfc_1034 else r'$'}",
+            + rf"[a-z]{r'.?$' if rfc_1034 else r'$'}",
             value.encode("idna").decode("utf-8"),
             re.IGNORECASE,
         )
-    except UnicodeError:
-        return False
+    except UnicodeError as err:
+        raise UnicodeError(f"Unable to encode/decode {value}") from err
